%____copyright___="Copyright (C) 2022 A. Nolla, A.I. Muoz, E. Schiavi.
%____license____="GPL-3.0-only"

%Detalles sobre el empleo de los cdigos y ejercicios de aplicacin pueden
%encontrarse en las direcciones de los siguientes documentos elaborados por
%A. Nolla, A.I. Muoz, E. Schiavi:
%https://burjcdigital.urjc.es/handle/10115/20132
%https://burjcdigital.urjc.es/handle/10115/20134
%As como en el fichero Readme

% La mayor parte de los cdigos de la coleccin presentada en esta librera
%son adaptaciones de los publicados en el libro "Clculo cientfico con 
% Matlab y Octave" de A. Quarteroni y F. Saliery, 
% que se pueden obtener en https://mox.polimi.it/qs/.



function [sol,itera]=metnewton1ec(fecu,dfecu,x0,errorper,maxitera)

%   Este cdigo obtiene una solucin aproximada de una raz
%   de la ecuacin fecu=0, con el mtodo de Newton-Raphson
%   tomando como semilla x0
%   fecu es la funcin que definie la ecuacin de la cual queremos obtener
%   una raz, dfecu es su derivada primera
%   sol es la solucin nnumrica obtenida
%   errorper es el mximo error permitido en la aproximacin
%   maxitera es el nmero mximo de iteraciones permitidas
%   itera es el nmero de iteracioes realizadas por el esquema para
%   alcanzar un error por debajo del error cometido. El error se mide con
%   el valor absoluto de fecu(x)/dfecu(x)

x = x0;
fx = fecu(x);
dfx = dfecu(x);
itera = 0; medidaerror = errorper+1;
while medidaerror >= errorper & itera < maxitera
   itera = itera + 1;      
   medidaerror = - fx/dfx;
   x = x + medidaerror;
   medidaerror= abs(medidaerror);
   fx = fecu(x);
   dfx = dfecu(x);
end
if (itera==maxitera & medidaerror > errorper)
  fprintf(['el mtodo no converge por alcanzar el nmero mximo  ',...
   'de iteraciones permitidas ',...
   'sin llegar a un error por debajo del error mximo permitido']);
end
sol = x;
return
