%____copyright___="Copyright (C) 2022 A. Nolla, A.I. Muoz, E. Schiavi."
%____license____="GPL-3.0-only"

%Detalles sobre el empleo de los cdigos y ejercicios de aplicacin pueden
%encontrarse en las direcciones de los siguientes documentos elaborados por
%A. Nolla, A.I. Muoz, E. Schiavi:
%https://burjcdigital.urjc.es/handle/10115/20132
%https://burjcdigital.urjc.es/handle/10115/20134
%As como en el fichero Readme

% La mayor parte de los cdigos de la coleccin presentada en esta librera
%son adaptaciones de los publicados en el libro "Clculo cientfico con 
% Matlab y Octave" de A. Quarteroni y F. Saliery, 
% que se pueden obtener en https://mox.polimi.it/qs/.



function [x,uf]=ecucalor(C,intespacio,intiempo,pasosespacio, ...
               pasostiempo,theta,u0,cc,f)
%este cdigo resuelve la ecuacin del calor en una dimesin espacial
%utilizando un theta-mtodo para la discretizacin temporal
%du/dt=C (d^2 u/dx^2) +f  (t0,T)x(a,b)
%intespacio=(a,b) intiempo=(t0,T)
%la condicin inicial se deota por u0
%la condicin de contorno tipo Dirichlet, la denotamos por cc
%theta es el valor de theta empleado en el theta mtodo
%theta=0 es para Euler explcito
%theta=1 es para Euler implcito
%theta=0.5 para Crank-Nicolson
%pasosespacio es el nmero de pasos en espacio
%pasostiempo es el nmero de pasos en tiempo
%x es el vector que contiene los nodos en espacio
%uf contiene los valores numricos obtenidos en t=T

h  = (intespacio(2)-intespacio(1))/pasosespacio;
dt = (intiempo(2)-intiempo(1))/pasostiempo;
N = pasosespacio+1;%nmero de nodos en espacio
%perparamos la matriz de coeficientes, que variar dependiendo del
%theta-mtodo empleado
e =ones(N,1); D = spdiags([-e 2*e -e],[-1,0,1],N,N);
I = speye(N); M = I+C*dt*theta*D/h^2;
Mn = I-C*dt*(1-theta)*D/h^2;
 M(1,:) = 0;
 M(1,1) = 1; M(N,:) = 0; M(N,N) = 1;
 x =linspace(intespacio(1),intespacio(2),N);
 x = x'; fn = feval(f,intiempo(1),x); 
 un =feval(u0,x);
 [L,U]=lu(M); 
 for t = intiempo(1)+dt:dt:intiempo(2)
    fn1 = feval(f,t,x);
    %lado derecho
    rhs = Mn*un+dt*(theta*fn1+(1-theta)*fn);
    temp = feval(cc,t,[intespacio(1),intespacio(2)]);
    rhs([1,N]) = temp;
    %el sistema se resuleve utilizando una factorizacin LU
    u = L\rhs;
    u = U\u;
    fn = fn1;
    un = u;
 
 end
uf=u;
return
